/*******************************************************************************
 * Copyright (c) 2015, 2016 CNRS, IRIT, IRT AESE (IRT Saint Exupéry).
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the LGPL 3.0
 * which accompanies this distribution, and is available at
 * https://www.gnu.org/licenses/lgpl-3.0-standalone.html
 *
 * Contributors:
 *     Faiez Zalila - initial API and implementation and/or initial documentation
 *******************************************************************************/
/**
 */
package com.irtsaintexupery.fiacre.instance.fiacreinstance.provider;


import fr.irit.fiacre.xtext.fiacre.FiacreFactory;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;

import org.eclipse.emf.common.util.ResourceLocator;

import org.eclipse.emf.ecore.EStructuralFeature;

import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ItemProviderAdapter;
import org.eclipse.emf.edit.provider.ViewerNotification;

import com.irtsaintexupery.fiacre.instance.fiacreinstance.FiacreinstancePackage;
import com.irtsaintexupery.fiacre.instance.fiacreinstance.VariableDeclarationInstance;

/**
 * This is the item provider adapter for a {@link com.irtsaintexupery.fiacre.instance.fiacreinstance.VariableDeclarationInstance} object.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class VariableDeclarationInstanceItemProvider 
	extends ItemProviderAdapter
	implements
		IEditingDomainItemProvider,
		IStructuredItemContentProvider,
		ITreeItemContentProvider,
		IItemLabelProvider,
		IItemPropertySource {
	/**
	 * This constructs an instance from a factory and a notifier.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public VariableDeclarationInstanceItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}

	/**
	 * This returns the property descriptors for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);

			addVariableDeclarationPropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}

	/**
	 * This adds a property descriptor for the Variable Declaration feature.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void addVariableDeclarationPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add
			(createItemPropertyDescriptor
				(((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
				 getResourceLocator(),
				 getString("_UI_VariableDeclarationInstance_variableDeclaration_feature"),
				 getString("_UI_PropertyDescriptor_description", "_UI_VariableDeclarationInstance_variableDeclaration_feature", "_UI_VariableDeclarationInstance_type"),
				 FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__VARIABLE_DECLARATION,
				 true,
				 false,
				 true,
				 null,
				 null,
				 null));
	}

	/**
	 * This specifies how to implement {@link #getChildren} and is used to deduce an appropriate feature for an
	 * {@link org.eclipse.emf.edit.command.AddCommand}, {@link org.eclipse.emf.edit.command.RemoveCommand} or
	 * {@link org.eclipse.emf.edit.command.MoveCommand} in {@link #createCommand}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Collection<? extends EStructuralFeature> getChildrenFeatures(Object object) {
		if (childrenFeatures == null) {
			super.getChildrenFeatures(object);
			childrenFeatures.add(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE);
		}
		return childrenFeatures;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EStructuralFeature getChildFeature(Object object, Object child) {
		// Check the type of the specified child object and return the proper feature to use for
		// adding (see {@link AddCommand}) it as a child.

		return super.getChildFeature(object, child);
	}

	/**
	 * This returns VariableDeclarationInstance.gif.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/VariableDeclarationInstance"));
	}

	/**
	 * This returns the label text for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String getText(Object object) {
		return getString("_UI_VariableDeclarationInstance_type");
	}
	

	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached
	 * children and by creating a viewer notification, which it passes to {@link #fireNotifyChanged}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);

		switch (notification.getFeatureID(VariableDeclarationInstance.class)) {
			case FiacreinstancePackage.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), true, false));
				return;
		}
		super.notifyChanged(notification);
	}

	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children
	 * that can be created under this object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createValueExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createIdentifierExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createRecordExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createArrayExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createQueueExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createLiteralExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createBooleanLiteral()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createNaturalLiteral()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createConstructionExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createArrayAccessExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createRecordAccessExpression()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createTrueLiteral()));

		newChildDescriptors.add
			(createChildParameter
				(FiacreinstancePackage.Literals.VARIABLE_DECLARATION_INSTANCE__CURRENT_VALUE,
				 FiacreFactory.eINSTANCE.createFalseLiteral()));
	}

	/**
	 * Return the resource locator for this item provider's resources.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator() {
		return FiacreInstanceEditPlugin.INSTANCE;
	}

}
